#
# Find HDF5 includes and library
#
# HDF5
# It can be found at:
#     http://amas.web.psi.ch/tools/HDF5/index.html
#
# HDF5_INCLUDE_DIR - where to find hdf5.h
# HDF5_LIBRARY     - qualified libraries to link against.
# HDF5_FOUND       - do not attempt to use if "no" or undefined.

SET(CMAKE_FIND_LIBRARY_SUFFIXES .lib .a ${CMAKE_FIND_LIBRARY_SUFFIXES})

FIND_PATH(HDF5_INCLUDE_DIR hdf5.h
    HINTS $ENV{HDF5_INCLUDE_PATH} $ENV{HDF5_INCLUDE_DIR} $ENV{HDF5_PREFIX}/include $ENV{HDF5_DIR}/include $ENV{HDF5}/include
    PATHS ENV C_INCLUDE_PATH
)

FIND_LIBRARY(HDF5_LIBRARY libhdf5.a
    HINTS $ENV{HDF5_LIBRARY_PATH} $ENV{HDF5_LIBRARY_DIR} $ENV{HDF5_PREFIX}/lib $ENV{HDF5_DIR}/lib $ENV{HDF5}/lib
    PATHS ENV LIBRARY_PATH
)


IF(HDF5_INCLUDE_DIR AND HDF5_LIBRARY)
    SET( HDF5_FOUND "YES" )
ENDIF(HDF5_INCLUDE_DIR AND HDF5_LIBRARY)

IF (HDF5_FOUND)
   IF (NOT HDF5_FIND_QUIETLY)
      MESSAGE(STATUS "Found HDF5 library: ${HDF5_LIBRARY}")
      MESSAGE(STATUS "Found HDF5 include dir: ${HDF5_INCLUDE_DIR}")
   ENDIF (NOT HDF5_FIND_QUIETLY)
ELSE (HDF5_FOUND)
   IF (HDF5_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find HDF5!")
   ENDIF (HDF5_FIND_REQUIRED)
ENDIF (HDF5_FOUND)
